/*
 * Created on 6 juil. 2005
 *
 */
package fr.umlv.tatoo.cc.common.generator;

import java.io.File;
import java.util.Map;


import fr.umlv.tatoo.cc.common.generator.impl.Motocity;
import fr.umlv.tatoo.cc.common.generator.impl.Velocity;
import fr.umlv.tatoo.cc.common.main.Alias;
import fr.umlv.tatoo.cc.common.main.AliasPrototype;

/** Generate enums that represents at runtimn
 *  productions, terminals, non-terminals, rules and parser tables
 *  using respectively an array of
 *  {@link fr.umlv.tatoo.cc.parser.grammar.ProductionDecl ProductionDecl},
 *  {@link fr.umlv.tatoo.cc.parser.grammar.TerminalDecl TerminalDecl},
 *  {@link fr.umlv.tatoo.cc.parser.grammar.NonTerminalDecl NonTerminalDecl},
 *  {@link fr.umlv.tatoo.cc.lexer.lexer.RuleDecl RuleDecl},
 *  {@link fr.umlv.tatoo.cc.parser.table.ParserTableDecl ParserTableDecl}.
 *  
 *  
 * @author remi
 *
 */
public class Generator {
  
  public Generator(File sourceDir,Class<?> clazz) {
    this.clazz = clazz;
    this.sourceDir = sourceDir;
  }
  
  public void generate(Map<String,?> root,Map<AliasPrototype,? extends Alias> aliasMap,AliasPrototype prototype) throws GeneratorException {
    Alias alias=aliasMap.get(prototype);
    if (alias.generate())
      generate(root,prototype.getDefaultTypeName(),alias.getType());
  }
  
  private static final Velocity VELOCITY = new Velocity();
  private static final Motocity MOTOCITY = new Motocity();
  
  /** Generates the source code of a type using a template.
   *  
   * @param map a map containg properties.
   * @param templateName the name of the velocity template.
   * @param type the generated type.
   * @throws GeneratorException 
   */
  protected void generate(Map<String,?> map,String templateName,Type type) throws GeneratorException {
    String packageName=type.getPackageName();
    String simpleName=type.getSimpleName();
    
    String pathName=packageName.replace('.',File.separatorChar);
    File path=new File(sourceDir,pathName);
    File file=new File(path,simpleName+".java");
    path.mkdirs();
    
    if (clazz.getResource(templateName+".mc")!=null)
      MOTOCITY.generate(map, templateName, file, sourceDir, clazz);
    else
      VELOCITY.generate(map, templateName, file, sourceDir, clazz);
  }
  
  private final Class<?> clazz;
  private final File sourceDir;
}
