package fr.umlv.tatoo.cc.parser.table;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.runtime.RuntimeInstance;

import fr.umlv.tatoo.cc.common.log.Info;
import fr.umlv.tatoo.cc.common.util.MultiMap;
import fr.umlv.tatoo.cc.common.velocity.VelocityLogger;
import fr.umlv.tatoo.cc.parser.grammar.Grammar;
import fr.umlv.tatoo.cc.parser.grammar.GrammarSets;
import fr.umlv.tatoo.cc.parser.grammar.NonTerminalDecl;
import fr.umlv.tatoo.cc.parser.grammar.ProductionDecl;
import fr.umlv.tatoo.cc.parser.grammar.TerminalDecl;
import fr.umlv.tatoo.cc.parser.grammar.VariableDecl;
import fr.umlv.tatoo.cc.parser.parser.AcceptActionDecl;
import fr.umlv.tatoo.cc.parser.parser.BranchActionDecl;
import fr.umlv.tatoo.cc.parser.parser.EnterActionDecl;
import fr.umlv.tatoo.cc.parser.parser.ErrorActionDecl;
import fr.umlv.tatoo.cc.parser.parser.ExitActionDecl;
import fr.umlv.tatoo.cc.parser.parser.ReduceActionDecl;
import fr.umlv.tatoo.cc.parser.parser.ShiftActionDecl;
import fr.umlv.tatoo.cc.parser.parser.SimpleActionDeclVisitor;
import fr.umlv.tatoo.cc.parser.parser.VersionedActionDecl;

/**
 * @author Gilles
 * @version $Revision: 1431 $ $Date: 2007-11-23 18:19:25 +0100 (ven, 23 nov 2007) $
 */
public class TableWriter {

  
  public static <I extends NodeItem<I>> void dumpTable(File log,
      Grammar grammar,
      GrammarSets grammarSets,
      NodeFactory<I> nodeFactory,
      Map<NodeDecl<I>, ? extends MultiMap<TerminalDecl,?>> table,
      MultiMap<NodeDecl<I>,?> branch,
      Map<NodeDecl<I>, ? extends Map<NonTerminalDecl,NodeDecl<I>>> buildedGotos) {
    VelocityContext root = new VelocityContext();
    root.put("mapGetter", MapGetter.getInstance());
    root.put("grammar", grammar);
    root.put("grammarSets",grammarSets);
    root.put("nodeFactory", nodeFactory);
    root.put("table",table);
    root.put("branch",new HashMap<NodeDecl<I>,Object>(branch));
    root.put("buildedGotos", buildedGotos);
    root.put("displayVisitor", new SimpleActionDeclVisitor<String>() {
      @Override
      public String visit(ErrorActionDecl error) {
        return "error\u00a0"+error.getMessage();
      }
      @Override
      public String visit(BranchActionDecl error) {
        return "branch\u00a0"+error.getMessage();
      }
      @Override
      public String visit(ReduceActionDecl reduce) {
        ProductionDecl prod = reduce.getProduction();
        StringBuilder b = new StringBuilder("reduce\u00a0by\u00a0");
        b.append(prod.getLeft()).append("\u00a0::=\u00a0");
        if (prod.getRight().isEmpty())
          b.append("\u025b");
        else {
          for(VariableDecl v:prod.getRight())
            b.append(v).append("\u00a0");
          b.setLength(b.length()-1);
        }
        return b.toString();
      }
      @Override
      public String visit(ShiftActionDecl shift) {
        return String.format("shift\u00a0to\u00a0<a href=\"#%1$s\">%1$s</a>",shift.getState());
      }
      @Override
      public String visit(AcceptActionDecl accept) {
        return "accept";
      }
      @Override
      public String visit(VersionedActionDecl versioned) {
        throw new UnsupportedOperationException("invalid action type");
      }
      @Override
      public String visit(EnterActionDecl enter) {
        return "enter\u00a0"+enter.getBranchingTerminal().getId();
      }
      @Override
      public String visit(ExitActionDecl exit) {
        return "exit\u00a0"+exit.getId();
      }
    });
    RuntimeInstance ri = new RuntimeInstance();
    ri.setProperty("runtime.log.logsystem", new VelocityLogger());
    ri.setProperty("file.resource.loader.class", "fr.umlv.tatoo.cc.common.velocity.ClassResourceLoader");
    ri.setProperty("file.resource.loader.resourceClass", TableWriter.class);
    try {
      ri.init();
      Writer out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(log),"UTF-8"));
      Template template = ri.getTemplate("debugToXHTML.vm", "UTF-8");
      template.merge(root, out);
      out.close();
      
      Info.info("Grammar table wrote to log file: %s",log).report();
    } catch(Exception e) {
      Info.error("error while writing table to log file: %s",log).
        cause(e).report();
    }
  }
}