package fr.umlv.tatoo.motocity.compiler;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import java.util.ArrayList;
import java.util.Collections;

import javax.tools.JavaCompiler;
import javax.tools.StandardJavaFileManager;
import javax.tools.StandardLocation;
import javax.tools.ToolProvider;
import javax.tools.JavaCompiler.CompilationTask;
import javax.tools.JavaFileObject.Kind;

public class MemoryCompiler {
  private MemoryCompiler() {
    // this class is only a collection of static methods
  }
  
  /**
   * Return the classpath element with contains the bytecode of a class
   * @param clazz the seeked class
   * @return a file which is the directory or the jar archive which is the root of the class location
   */
  public static File getPath(Class<?> clazz) {
    URL url = clazz.getResource(clazz.getSimpleName()+".class");
    if (url.getProtocol().equals("jar")) {
      String file = url.getPath();
      int bang = file.indexOf('!');
      file=file.substring(0,bang);
      file=file.substring(5); // trim file:
      file=file.replace('/',File.separatorChar);
      File classPath=new File(file);
      if (!classPath.exists())
        throw new IllegalStateException("Can't find jar archive (tried "+classPath+")");
      return classPath;
    }
    else {
      String file=url.getPath();
      file=file.substring(0,file.length()-clazz.getName().length()-6);
      File classPath=new File(file);
      if (!classPath.exists())
        throw new IllegalStateException("Can't classes (tried "+classPath+")");
      return classPath;
    }
  }
  
  public static Class<?> compile(String className,byte[] array,Iterable<? extends File> classPath) throws IOException {
    return compile(className,array,MemoryCompiler.class.getClassLoader(),classPath);
  }
  
  public static Class<?> compile(String className,byte[] array,ClassLoader parentLoader,Iterable<? extends File> classPath) throws IOException {
    JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
    if (compiler==null)
      throw new IllegalStateException("no compiler available (maybe java is from jre and not jdk, by default under windows)");
    StandardJavaFileManager standardFileManager = compiler.getStandardFileManager(null, null, null);
    if (classPath!=null) {
      ArrayList<File> list = new ArrayList<File>();
      for(File f : standardFileManager.getLocation(StandardLocation.CLASS_PATH))
        list.add(f);
      for(File f : classPath)
        list.add(f);
      standardFileManager.setLocation(StandardLocation.CLASS_PATH, list);
    }
    MemoryFileManager memoryFileManager = new MemoryFileManager(standardFileManager,parentLoader);
    CompilationTask task = compiler.getTask(null, memoryFileManager, null, null,
        null, Collections.singleton(new MemoryJavaFileObject(className,Kind.SOURCE,StandardLocation.SOURCE_PATH,array)));
    if (!task.call())
      return null;
    try {
      return memoryFileManager.getClassLoader(parentLoader).loadClass(className);
    } catch (ClassNotFoundException e) {
      return null;
    }
  }
}
